VPATH = $(OFILE_DIR)

.SUFFIXES: .m .o .c .psw .pswm .subproj .clean .install .installsrc .depend
.c.o:
	$(CC) $(_CFLAGS) -c $*.c -o $(OFILE_DIR)/$*.o
.m.o:
	$(CC) $(_CFLAGS) $(OBJCFLAGS) -c $*.m -o $(OFILE_DIR)/$*.o
.psw.h:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.c $*.psw
.psw.o:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.c $*.psw
	$(CC) $(_CFLAGS) -c $*.c -o $(OFILE_DIR)/$*.o
.pswm.h:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.m $*.pswm
.pswm.o:
	$(PSWRAP) $(PSWFLAGS) -a -h $*.h -o $*.m $*.pswm
	$(CC) $(_CFLAGS) $(OBJCFLAGS) -c $*.m -o $(OFILE_DIR)/$*.o
.subproj.o:
	(cd $*.subproj; $(MAKE) ofiles "NAME=$*" "_CFLAGS=$(_CFLAGS)" \
		"OFILE_DIR = $(OFILE_DIR)" "LIBS=")
.subproj.h:
	(cd $*.subproj; $(MAKE) headers "NAME=$*")
.subproj.clean:
	(cd $*.subproj; $(MAKE) clean "NAME=$*" "OFILE_DIR=$(OFILE_DIR)")
.subproj.depend:
	(cd $*.subproj; $(MAKE) depend "NAME=$*")
.subproj.installsrc:
	(cd $*.subproj; $(MAKE) installsrc \
		"NAME=$*" "SRCROOT=$(SRCROOT)/$*.subproj" \
		"SOURCEMODE=$(SOURCEMODE)")

$(NAME).installsrc:: SRCROOT $(SRCROOT)
	tar cf - $(SRCFILES) | (cd $(SRCROOT); tar xf -)
	@supportfiles=""; \
	for i in $(SUPPORTFILES) none ; do \
	    if [ -r $$i -a ! -r $(SRCROOT)/$$i ] ; then \
		supportfiles="$$supportfiles $$i"; \
	    fi; \
	done ; \
	echo "tar cf - $$supportfiles | (cd $(SRCROOT); tar xf -)"; \
	tar cf - $$supportfiles | (cd $(SRCROOT); tar xf -); \
	(cd $(SRCROOT); \
	    echo "chmod files to $(SOURCEMODE)"; \
	    chmod $(SOURCEMODE) $(SRCFILES) $$supportfiles)

$(NAME).list:
	@enscript $(CLASSES) $(SRCFILES) $(DATA_FILES) $(OTHER_PRODUCT_DEPENDS)
	
DATE:
	@if [ -n "${$@}" ]; then exit 0; \
	else echo Must define $@; exit 1; fi
	
DATE1:
	@if [ -n "${$@}" ]; then exit 0; \
	else echo Must define $@; exit 1; fi
	
DATE2:
	@if [ -n "${$@}" ]; then exit 0; \
	else echo Must define $@; exit 1; fi

TARFILES = $(SRCFILES) $(CLASSES) $(DATA_FILES) Makefile \
			$(HELP_FILES) $(OTHER_PRODUCT_DEPENDS)

$(NAME).tar:: DATE
	@tar cf - $(TARFILES) | \
	(cd $(TARFILEDIR) ; compress > $(NAME).$(DATE).tar.Z )
	
$(NAME).untar:: DATE
	@(cd $(TARFILEDIR) ; zcat $(NAME).$(DATE).tar.Z ) | \
	tar -xf - 
	
$(NAME).tarlist:: DATE
	@(cd $(TARFILEDIR) ; zcat $(NAME).$(DATE).tar.Z ) | \
	tar -tf -
	
$(NAME).xtract:: DATE
	@(cd $(TARFILEDIR) ; zcat $(NAME).$(DATE).tar.Z ) | \
	tar -xf - $(LIST)
	
$(NAME).tardir:
	@cd $(TARFILEDIR) ; /bin/ls $(NAME).*.tar.Z 
	
$(NAME).tardiff:: DATE1 DATE2
	@/bin/rm -rf /tmp/tar.$(DATE1); \
	/bin/rm -rf /tmp/tar.$(DATE2); \
	/bin/rm  /tmp/tarlist ; \
	mkdir /tmp/tar.$(DATE1); \
	mkdir /tmp/tar.$(DATE2); \
	(cd $(TARFILEDIR); zcat $(NAME).$(DATE1).tar.Z ) | \
	(cd /tmp/tar.$(DATE1); tar -xf - ) ; \
	(cd $(TARFILEDIR); zcat $(NAME).$(DATE2).tar.Z ) | \
	(cd /tmp/tar.$(DATE2); tar -xf - ) ; \
	(cd /tmp/tar.$(DATE1); ls $(TARFILES) ) > /tmp/tarlist ; \
	(cd /tmp/tar.$(DATE2); ls $(TARFILES) ) >> /tmp/tarlist ; \
	for i in `cat /tmp/tarlist | sort -u` ; do	\
	diff /tmp/tar.$(DATE1)/$i /tmp/tar.$(DATE2)/$i ; \
	done 
	
$(NAME).test:
	@$(NAME) | diff - $(TEST_OUT)

$(NAME).lib:: all
		@/bin/rm lib$(NAME).a; \
		for i in `ls $(OFILE_DIR)` ; do \
	    	ld -r -x $(OFILE_DIR)/$$i ; \
			mv a.out $(OFILE_DIR)/$$i ; \
			ar q lib$(NAME).a $(OFILE_DIR)/$$i; \
		done ; \
		ranlib lib$(NAME).a

$(NAME).install:
	@if [ ${APPWRAPPER} ] ; then \
	    echo install $(INSTALLFLAGS) $(NAME).app $(DSTROOT)$(INSTALLDIR); \
	    mkdirs $(DSTROOT)$(INSTALLDIR)/$(NAME).app ; \
	    chmod 755 $(DSTROOT)$(INSTALLDIR)/$(NAME).app ; \
	    install $(INSTALLFLAGS) $(NAME) $(DSTROOT)$(INSTALLDIR)/$(NAME).app; \
	    if [ ${NOMACHO} ] ; then  \
		install -c -m 644 $(INTERFACES) $(TIFFFILES) $(EPSFILES) $(SNDFILES) \
			$(DSTROOT)$(INSTALLDIR)/$(NAME).app ; \
	    fi ; \
	else \
	    echo install $(INSTALLFLAGS) $(NAME) $(DSTROOT)$(INSTALLDIR) ; \
	    install $(INSTALLFLAGS) $(NAME) $(DSTROOT)$(INSTALLDIR) ; \
	fi;

$(NAME).clean::
	@if [ ! -w . ] ; then echo '***' `pwd` 'not writable'; exit 1 ; fi
	-rm -rf $(GARBAGE)

$(OFILE_DIR):
	mkdirs $(OFILE_DIR)

SRCROOT:
	@if [ -n "${$@}" ]; then exit 0; \
	else echo Must define $@; exit 1; fi

$(SRCROOT)::
	-rm -rf $(SRCROOT)
	mkdirs $(SRCROOT)
	chmod 755 $(SRCROOT)

diff: SRCROOT
	@echo Compare directories: `pwd` and $(SRCROOT)
	@csh -f -c "diff -r `pwd` $(SRCROOT); exit 0"	


Makefile.dependencies:: $(CLASSES) $(MFILES) $(CFILES)
	-rm -f Makefile.dependencies
	$(CC) -MM $(CFLAGS) $(CLASSES) $(MFILES) $(CFILES) | \
	    awk '{ if ($$1 != prev) { if (rec != "") print rec; \
		rec = $$0; prev = $$1; } \
		else { if (length(rec $$2) > 78) { print rec; rec = $$0; } \
		else rec = rec " " $$2 } } \
		END { print rec }' > Makefile.dependencies

$(DSTROOT)$(INSTALLDIR):
	mkdirs $(DSTROOT)$(INSTALLDIR)


help::
	@echo '  diff SRCROOT=somepath'
